<?php declare(strict_types=1);

namespace App\Service;

error_reporting(error_level: E_ALL);

use App\Controller\ConfigController;
use App\Controller\CLIController;
use App\Controller\DomainController;
use App\Controller\RequestController;
use App\Repository\DomainRepository;
use App\Repository\DynDNSRepository;
use App\Service\ApiClient;
use DI\Container;
use DI\ContainerBuilder;
use DI\DependencyException;
use DI\NotFoundException;
use Exception;
use Monolog\Formatter\LineFormatter;
use Monolog\Handler\StreamHandler;
use Monolog\Level;
use Monolog\Logger;
use OpenApi\Attributes as OA;
use function DI\autowire;

#[OA\Info(version: '1.0.9', title: 'bindAPI')]
class BindAPI
{
    private Logger $logger;
    private Container $container;

    public function __construct(bool $quiet)
    {
        // init the logger
        $dateFormat = "Y:m:d H:i:s";
        $output = "%datetime% %channel%.%level_name% %message%\n"; // %context% %extra%
        $formatter = new LineFormatter(format: $output, dateFormat: $dateFormat);

        $debug = (new ConfigController(quiet: $quiet))->getConfig(configKey: 'debug');
        if ($debug) {
            $stream = new StreamHandler(stream: dirname(path: __DIR__, levels: 2) . '/var/log/bindAPI.debug', level: Level::Debug);
        } else {
            $stream = new StreamHandler(stream: dirname(path: __DIR__, levels: 2) . '/var/log/bindAPI.info', level: Level::Info);
        }
        $stream->setFormatter(formatter: $formatter);

        $this->logger = new Logger(name: 'bindAPI');
        $this->logger->pushHandler(handler: $stream);
        $this->logger->debug(message: 'bindAPI started');


        $containerBuilder = new ContainerBuilder();
        $containerBuilder->addDefinitions([
            ApiClient::class => autowire(),
            ConfigController::class => autowire()
                ->constructorParameter(parameter: 'quiet', value: $quiet),
            CLIController::class => autowire()
                ->constructorParameter(parameter: 'logger', value: $this->logger)
                ->constructorParameter(parameter: 'quiet', value: $quiet),
            DomainController::class => autowire()
                ->constructorParameter(parameter: 'logger', value: $this->logger)
                ->constructorParameter(parameter: 'quiet', value: $quiet),
            DomainRepository::class => autowire()
                ->constructorParameter(parameter: 'logger', value: $this->logger),
            DynDnsRepository::class => autowire()
                ->constructorParameter(parameter: 'logger', value: $this->logger),
            RequestController::class => autowire()
                ->constructorParameter(parameter: 'logger', value: $this->logger)
        ]);
        $this->container = $containerBuilder->build();
    }


    /**
     * @throws DependencyException
     * @throws NotFoundException
     */
    public function runCommand(array $arguments): void
    {
        $this->logger->debug(message: 'runCommand()');
        $cliController = $this->container->get(name: CLIController::class);
        $cliController->runCommand(arguments: $arguments);
    }


    /**
     * @throws DependencyException
     * @throws NotFoundException
     */
    public function handleRequest(string $requestMethod, array $uri): void
    {
        $this->logger->debug(message: 'handleRequest()');
        $requestController = $this->container->get(name: RequestController::class);
        $requestController->handleRequest(requestMethod: $requestMethod, uri: $uri);
    }
}
